"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setup = void 0;
const nest_librarian_1 = require("@jchpro/nest-librarian");
const path = require("path");
const config_1 = require("../config");
const collector_1 = require("../templates/collector");
const promises_1 = require("fs/promises");
async function setup(projectName, options) {
    // Get Nest project workspace
    const workspace = await nest_librarian_1.Project.locate();
    const nest = workspace.config;
    const pkg = await workspace.getPackageJson();
    // Check project
    if (!(projectName in nest.projects)) {
        console.error(`Project "${projectName}" not found in Nest.js CLI config`);
        process.exit(1);
    }
    const nestProject = nest.projects[projectName];
    if (nestProject.type !== 'application') {
        console.error(`Project "${projectName}" must be an application!`);
        process.exit(1);
    }
    // Read config
    const config = pkg.mongoMigrate ?? { projects: {} };
    // Add project
    const { collector, directory } = options;
    const project = {
        collector,
        directory
    };
    config.projects[projectName] = project;
    if (options.default) {
        config.default = projectName;
    }
    pkg.mongoMigrate = config;
    await (0, nest_librarian_1.writeJsonObject)(workspace.packageJsonPath, pkg);
    console.log(`Added project "${projectName}" to the migrations config.`);
    // Create directory
    const projectSrcDir = path.resolve(workspace.rootDir, nestProject.sourceRoot);
    const migrationsDir = path.resolve(projectSrcDir, project.directory);
    await (0, nest_librarian_1.createPaths)([migrationsDir]);
    console.log(`Created directory: ${migrationsDir}`);
    // Create collector file
    const collectorPath = path.resolve(projectSrcDir, project.collector);
    if (await (0, nest_librarian_1.exists)(collectorPath)) {
        console.log('Skipping creation of collector file, already exists.');
        return;
    }
    await (0, promises_1.writeFile)(collectorPath, (0, collector_1.collectorTemplate)(config_1.LIB_NAME), 'utf-8');
    console.log(`Created collector file: ${collectorPath}`);
}
exports.setup = setup;
