"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var MigrateService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigrateService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const changelog_lock_schema_1 = require("../schemas/changelog-lock.schema");
const changelog_schema_1 = require("../schemas/changelog.schema");
/**
 * Used internally by the migration mechanism, for use only in `main.ts`.
 */
let MigrateService = MigrateService_1 = class MigrateService {
    constructor(connection, migrations, changelogModel, changelogLockModel) {
        this.connection = connection;
        this.migrations = migrations;
        this.changelogModel = changelogModel;
        this.changelogLockModel = changelogLockModel;
        this.logger = new common_1.Logger(MigrateService_1.name);
    }
    async acquireLock() {
        const lock = await this.changelogLockModel.findOne().exec();
        if (!lock) {
            const result = await this.changelogLockModel.create(this.lockData());
            return !!result;
        }
        if (lock.locked) {
            return false;
        }
        const result = await this.changelogLockModel.updateOne({}, {
            $set: this.lockData()
        });
        return result.modifiedCount === 1;
    }
    async releaseLock() {
        const result = await this.changelogLockModel.updateOne({}, {
            $set: { locked: false },
            $unset: { processId: true }
        });
        return result.modifiedCount === 1;
    }
    async logMigration(migration) {
        await this.changelogModel.create({
            identifier: migration.id,
            applied: new Date()
        });
    }
    getChangelog() {
        return this.changelogModel.find({}, {}, {
            sort: { applied: 1 }
        }).exec();
    }
    lockData() {
        return {
            locked: true,
            processId: process.pid
        };
    }
};
exports.MigrateService = MigrateService;
MigrateService.MIGRATIONS = 'MONGO_MIGRATIONS';
exports.MigrateService = MigrateService = MigrateService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectConnection)()),
    __param(1, (0, common_1.Inject)(MigrateService.MIGRATIONS)),
    __param(2, (0, mongoose_1.InjectModel)(changelog_schema_1.Changelog.name)),
    __param(3, (0, mongoose_1.InjectModel)(changelog_lock_schema_1.ChangelogLock.name)),
    __metadata("design:paramtypes", [mongoose_2.Connection, Array, mongoose_2.Model,
        mongoose_2.Model])
], MigrateService);
